@@ -1,12 +1,20 @@
 """Interface for creating a db-contrib-tool plugin."""
 
 import abc
+from enum import Enum
+
+
+class SubcommandResult(Enum):
+    """Subcommand result."""
+
+    SUCCESS = 0
+    FAIL = 1
 
 
 class Subcommand(abc.ABC):
     """A db-contrib-tool subcommand to execute."""
 
-    def execute(self):
+    def execute(self) -> SubcommandResult:
         """Execute the subcommand."""
         raise NotImplementedError()
 
