@@ -21,7 +21,7 @@ import sys
 import time
 from collections import OrderedDict
 from enum import Enum
-from typing import Any, Dict, List, NamedTuple
+from typing import IO, Any, AnyStr, Dict, List, NamedTuple, Optional
 
 import inject
 import requests
@@ -29,7 +29,13 @@ from tenacity import Retrying, retry_if_result, stop_after_delay, wait_fixed
 
 from db_contrib_tool.clients.io_client import IOClient
 from db_contrib_tool.plugin import PluginInterface, Subcommand, SubcommandResult
-from db_contrib_tool.symbolizer.mongo_log_parser_service import MongoLogParserService
+from db_contrib_tool.symbolizer.mongo_log_parser_service import (
+    BINARY_LOAD_ADDRESS_KEY,
+    BUILD_ID_KEY,
+    MongoLogParserService,
+    SoInfo,
+    TraceDoc,
+)
 from db_contrib_tool.utils.build_system_options import PathOptions
 from db_contrib_tool.utils.oauth import (
     Configs,
@@ -39,6 +45,9 @@ from db_contrib_tool.utils.oauth import (
 
 SUBCOMMAND = "symbolize"
 
+SYMBOLIZER_PATH_ENV = "MONGOSYMB_SYMBOLIZER_PATH"
+DEFAULT_SYMBOLIZER_PATH = "/opt/mongodbtoolchain/v4/bin/llvm-symbolizer"
+
 
 class CachedResults(object):
     """
@@ -139,7 +148,6 @@ class PathResolver(object):
         self.host = host or self.default_host
         self._cached_results = CachedResults(max_cache_size=cache_size)
         self.cache_dir = cache_dir or self.default_cache_dir
-        self.mci_build_dir = None
         self.client_credentials_scope = (
             client_credentials_scope or self.default_client_credentials_scope
         )
@@ -192,7 +200,7 @@ class PathResolver(object):
 
         self.http_client.headers.update({"Authorization": f"Bearer {credentials.access_token}"})
 
-        # write credentials to local file for further useage
+        # write credentials to local file for further usage
         with open(self.default_creds_file_path, "w") as cfile:
             cfile.write(
                 json.dumps(
@@ -278,19 +286,19 @@ class PathResolver(object):
                     for chunk in response.iter_content(chunk_size=2 * 1024 * 1024):
                         file.write(chunk)
         else:
-            print("File aready exists in cache")
+            print("File already exists in cache")
             exists_locally = True
         return path, exists_locally
 
-    def get_dbg_file(self, soinfo: dict) -> str or None:
+    def get_dbg_file(self, soinfo: SoInfo) -> Optional[str]:
         """
         To get path for given buildId.
 
-        :param soinfo: soinfo as dict
-        :return: path as string or None (if path not found)
+        :param soinfo: Information about process that printed backtrace.
+        :return: Path as string or None (if path not found)
         """
-        build_id = soinfo.get("buildId", "").lower()
-        version = soinfo.get("version")
+        build_id = soinfo.build_id.lower() if soinfo.build_id is not None else ""
+        version = soinfo.mongodb_version
         binary_name = "mongo"
         # search from cached results
         path = self.get_from_cache(build_id)
@@ -300,19 +308,9 @@ class PathResolver(object):
                 search_parameters = {"build_id": build_id}
                 if version:
                     search_parameters["version"] = version
+                print(f"Getting data from service... Search parameters: {search_parameters}")
                 response = self.http_client.get(f"{self.host}/find_by_id", params=search_parameters)
                 if response.status_code != 200:
-                    # if we could not find the path of binary, that might be system library.
-                    # we can try using frame's own `path` data.
-                    # symbolization can succeed only if that binary exists on local
-                    # machine (more specifically: in the given path).
-                    system_path = soinfo.get("path")
-                    if system_path:
-                        sys.stdout.write(
-                            f"Could not find path of binary from symbolizer web service. Trying to use the "
-                            f"provided path: {system_path}\n"
-                        )
-                        return system_path
                     sys.stderr.write(
                         f"Server returned unsuccessful status: {response.status_code}, "
                         f"response body: {response.text}\n"
@@ -380,7 +378,6 @@ class SymbolizerParameters(NamedTuple):
 
     * input_format: Input mongo log format.
     * output_format: Output format.
-    * src_dir_to_move: Src dir to move to /data/mci/{original_buildid}/src.
 
     * live: Whether it should enter live mode.
 
@@ -397,7 +394,6 @@ class SymbolizerParameters(NamedTuple):
 
     input_format: InputFormat
     output_format: OutputFormat
-    src_dir_to_move: str
 
     live: bool
 
@@ -414,21 +410,26 @@ class SymbolizerOrchestrator:
 
     @inject.autoparams()
     def __init__(
-        self, mongo_log_parser_service: MongoLogParserService, io_client: IOClient
+        self,
+        mongo_log_parser_service: MongoLogParserService,
+        io_client: IOClient,
+        dbg_path_resolver: PathResolver,
     ) -> None:
         """
         Initialize.
 
         :param mongo_log_parser_service: Service to parse backtraces in log files.
+        :param io_client: Client for working with input/output.
+        :param dbg_path_resolver: Debug symbols file path resolver.
         """
         self.mongo_log_parser_service = mongo_log_parser_service
         self.io_client = io_client
+        self.dbg_path_resolver = dbg_path_resolver
 
-    def substitute_stdin(self, resolver: PathResolver, params: SymbolizerParameters) -> None:
+    def substitute_stdin(self, params: SymbolizerParameters) -> None:
         """
         Accept stdin stream as source of logs and symbolize it.
 
-        :param resolver: Debug symbols path resolver.
         :param params: Symbolizer parameters.
         """
 
@@ -436,6 +437,7 @@ class SymbolizerOrchestrator:
         signal.signal(signal.SIGINT, signal.SIG_IGN)
 
         print("Live mode activated, waiting for input...")
+        global_base_addr_map = {}
         while True:
             backtrace_indicator = '{"backtrace":'
             line = sys.stdin.readline()
@@ -450,22 +452,18 @@ class SymbolizerOrchestrator:
             if backtrace_indicator in line:
                 backtrace_index = line.index(backtrace_indicator)
                 prefix = line[:backtrace_index]
-                backtrace = line[backtrace_index:]
-                trace_doc = json.loads(backtrace)
-                if not trace_doc["backtrace"]:
-                    print("Trace is empty, skipping...")
-                    continue
-                frames = self.symbolize_frames(
-                    trace_doc=trace_doc,
-                    dbg_path_resolver=resolver,
-                    symbolizer_path=params.symbolizer_path,
-                    dsym_hint=[],
-                    input_format=params.input_format,
-                    total_seconds_for_retries=params.total_seconds_for_retries,
+                trace_docs = self.mongo_log_parser_service.parse_log_lines(
+                    line, global_base_addr_map
                 )
-                print(prefix)
-                print("Symbolizing...")
-                self.classic_output(frames, sys.stdout, indent=2)
+                if len(trace_docs) == 0:
+                    print("Could not find json backtrace object in input...")
+                    continue
+                for trace_doc in trace_docs:
+                    frames = self.symbolize_frames(trace_doc, params)
+                    print(prefix)
+                    print("Symbolizing...")
+                    self.classic_output(frames, sys.stdout)
+                    global_base_addr_map = trace_doc.base_addr_map
                 print("Symbolization completed, waiting for input...")
             else:
                 print(line)
@@ -477,16 +475,9 @@ class SymbolizerOrchestrator:
         :param params: Symbolizer parameters.
         :return: Succeeded or not.
         """
-        resolver = PathResolver(
-            host=params.host,
-            cache_dir=params.cache_dir,
-            client_secret=params.client_secret,
-            client_id=params.client_id,
-        )
-
         if params.live:
             print("Entering live mode")
-            self.substitute_stdin(resolver, params)
+            self.substitute_stdin(params)
             return True
 
         log_lines = self.io_client.read_from_stdin()
@@ -496,9 +487,9 @@ class SymbolizerOrchestrator:
                 "e.g. `your/symbolization/command < /file/with/stacktrace`"
             )
 
-        trace_doc = self.mongo_log_parser_service.parse_log_lines(log_lines)
-        if trace_doc is None:
-            print("could not find json backtrace object in input", file=sys.stderr)
+        trace_docs = self.mongo_log_parser_service.parse_log_lines(log_lines)
+        if len(trace_docs) == 0:
+            print("Could not find json backtrace object in input", file=sys.stderr)
             return False
 
         output_fn = None
@@ -507,64 +498,35 @@ class SymbolizerOrchestrator:
         if params.output_format == OutputFormat.CLASSIC:
             output_fn = self.classic_output
 
-        frames = self.preprocess_frames_with_retries(resolver, trace_doc, params.input_format)
-
-        if params.src_dir_to_move and resolver.mci_build_dir is not None:
-            try:
-                os.makedirs(resolver.mci_build_dir)
-                os.symlink(
-                    os.path.join(os.getcwd(), params.src_dir_to_move),
-                    os.path.join(resolver.mci_build_dir, "src"),
-                )
-            except FileExistsError:
-                pass
-
-        frames = self.symbolize_frames(
-            frames=frames,
-            trace_doc=trace_doc,
-            dbg_path_resolver=resolver,
-            symbolizer_path=params.symbolizer_path,
-            dsym_hint=params.dsym_hint,
-            input_format=params.input_format,
-            total_seconds_for_retries=params.total_seconds_for_retries,
-        )
-        output_fn(frames, sys.stdout, indent=2)
+        for trace_doc in trace_docs:
+            frames = self.symbolize_frames(trace_doc, params)
+            output_fn(frames, sys.stdout, indent=2)
 
         return True
 
-    @staticmethod
-    def parse_input(trace_doc, dbg_path_resolver):
-        """Return a list of frame dicts from an object of {backtrace: list(), processInfo: dict()}."""
+    def produce_frames(self, trace_doc: TraceDoc) -> List[Dict[str, Any]]:
+        """
+        Return a list of frame dicts from an object of trace doc.
 
-        def make_base_addr_map(somap_list):
-            """Return map from binary load address to description of library from the somap_list.
-
-            The somap_list is a list of dictionaries describing individual loaded libraries.
-            """
-            return {so_entry["b"]: so_entry for so_entry in somap_list if "b" in so_entry}
-
-        base_addr_map = make_base_addr_map(trace_doc["processInfo"]["somap"])
-        version = trace_doc.get("processInfo", {}).get("mongodbVersion")
-        if version and "patch" in version:
-            version = version.split("-")[-1]
+        :param trace_doc: Traceback doc.
+        :return: List of traceback frames.
+        """
 
         frames = []
-        for frame in trace_doc["backtrace"]:
-            if "b" not in frame:
+        for frame in trace_doc.backtrace:
+            if BINARY_LOAD_ADDRESS_KEY not in frame:
                 print(
-                    f"Ignoring frame {frame} as it's missing the `b` field; See SERVER-58863 for discussions"
+                    f"Ignoring frame {frame} as it's missing the `{BINARY_LOAD_ADDRESS_KEY}` field;"
+                    f" See SERVER-58863 for discussions"
                 )
                 continue
-            soinfo = base_addr_map.get(frame["b"], {})
-            if version:
-                soinfo["version"] = version
-            elf_type = soinfo.get("elfType", 0)
-            if elf_type == 3:
+            soinfo = trace_doc.base_addr_map.get(frame[BINARY_LOAD_ADDRESS_KEY], SoInfo())
+            if soinfo.elf_type == 3:
                 addr_base = "0"
-            elif elf_type == 2:
-                addr_base = frame["b"]
+            elif soinfo.elf_type == 2:
+                addr_base = frame[BINARY_LOAD_ADDRESS_KEY]
             else:
-                addr_base = soinfo.get("vmaddr", "0")
+                addr_base = soinfo.vmaddr if soinfo.vmaddr is not None else "0"
             addr = int(addr_base, 16) + int(frame["o"], 16)
             # addr currently points to the return address which is the one *after* the call. x86 is
             # variable length so going backwards is difficult. However, llvm-symbolizer seems to do the
@@ -574,8 +536,8 @@ class SymbolizerOrchestrator:
             addr -= 1
             frames.append(
                 dict(
-                    path=dbg_path_resolver.get_dbg_file(soinfo),
-                    buildId=soinfo.get("buildId", None),
+                    path=self.dbg_path_resolver.get_dbg_file(soinfo),
+                    buildId=soinfo.build_id,
                     offset=frame["o"],
                     addr="0x{:x}".format(addr),
                     symbol=frame.get("s", None),
@@ -584,34 +546,34 @@ class SymbolizerOrchestrator:
         return frames
 
     def symbolize_frames(
-        self, trace_doc, dbg_path_resolver, symbolizer_path, dsym_hint, input_format, **kwargs
-    ):
-        """Return a list of symbolized stack frames from a trace_doc in MongoDB stack dump format."""
+        self, trace_doc: TraceDoc, params: SymbolizerParameters
+    ) -> List[Dict[str, Any]]:
+        """
+        Return a list of symbolized stack frames from a trace_doc in MongoDB stack dump format.
 
-        # Keep frames in kwargs to avoid changing the function signature.
-        frames = kwargs.get("frames")
-        if frames is None:
-            total_seconds_for_retries = kwargs.get("total_seconds_for_retries", 0)
-            frames = self.preprocess_frames_with_retries(
-                dbg_path_resolver,
-                trace_doc,
-                input_format,
-                total_seconds_for_retries=total_seconds_for_retries,
-            )
+        :param trace_doc: Traceback doc.
+        :param params: Symbolizer parameters.
+        :return: List of traceback frames.
+        """
 
+        frames = self.preprocess_frames_with_retries(
+            trace_doc,
+            params.input_format,
+            params.total_seconds_for_retries,
+        )
+
+        symbolizer_path = params.symbolizer_path
         if not symbolizer_path:
-            symbolizer_path_env = "MONGOSYMB_SYMBOLIZER_PATH"
-            default_symbolizer_path = "/opt/mongodbtoolchain/v4/bin/llvm-symbolizer"
-            symbolizer_path = os.environ.get(symbolizer_path_env)
+            symbolizer_path = os.environ.get(SYMBOLIZER_PATH_ENV)
             if not symbolizer_path:
                 print(
-                    f"Env value for '{symbolizer_path_env}' not found, using '{default_symbolizer_path}' "
+                    f"Env value for '{SYMBOLIZER_PATH_ENV}' not found, using '{DEFAULT_SYMBOLIZER_PATH}' "
                     f"as a defualt executable path."
                 )
-                symbolizer_path = default_symbolizer_path
+                symbolizer_path = DEFAULT_SYMBOLIZER_PATH
 
         symbolizer_args = [symbolizer_path]
-        for dh in dsym_hint:
+        for dh in params.dsym_hint:
             symbolizer_args.append("-dsym-hint={}".format(dh))
         symbolizer_process = subprocess.Popen(
             args=symbolizer_args,
@@ -621,34 +583,6 @@ class SymbolizerOrchestrator:
             stderr=sys.stdout,
         )
 
-        def extract_symbols(stdin):
-            """Extract symbol information from the output of llvm-symbolizer.
-
-            Return a list of dictionaries, each of which has fn, file, column and line entries.
-
-            The format of llvm-symbolizer output is that for every CODE line of input,
-            it outputs zero or more pairs of lines, and then a blank line. This way, if
-            a CODE line of input maps to several inlined functions, you can use the blank
-            line to find the end of the list of symbols corresponding to the CODE line.
-
-            The first line of each pair contains the function name, and the second contains the file,
-            column and line information.
-            """
-            result = []
-            step = 0
-            while True:
-                line = stdin.readline().decode()
-                if line == "\n":
-                    break
-                if step == 0:
-                    result.append({"fn": line.strip()})
-                    step = 1
-                else:
-                    file_name, line, column = line.strip().rsplit(":", 3)
-                    result[-1].update({"file": file_name, "column": int(column), "line": int(line)})
-                    step = 0
-            return result
-
         for frame in frames:
             if frame["path"] is None:
                 print("Path not found in frame:", frame)
@@ -656,29 +590,62 @@ class SymbolizerOrchestrator:
             symbol_line = "CODE {path:} {addr:}\n".format(**frame)
             symbolizer_process.stdin.write(symbol_line.encode())
             symbolizer_process.stdin.flush()
-            frame["symbinfo"] = extract_symbols(symbolizer_process.stdout)
+            frame["symbinfo"] = self.extract_symbols(symbolizer_process.stdout)
         symbolizer_process.stdin.close()
         symbolizer_process.wait()
         return frames
 
+    @staticmethod
+    def extract_symbols(stdout: Optional[IO[AnyStr]]) -> List[Dict[str, Any]]:
+        """
+        Extract symbol information from the output of llvm-symbolizer.
+
+        Return a list of dictionaries, each of which has fn, file, column and line entries.
+
+        The format of llvm-symbolizer output is that for every CODE line of input,
+        it outputs zero or more pairs of lines, and then a blank line. This way, if
+        a CODE line of input maps to several inlined functions, you can use the blank
+        line to find the end of the list of symbols corresponding to the CODE line.
+
+        The first line of each pair contains the function name, and the second contains the file,
+        column and line information.
+
+        :param stdout: Output of llvm-symbolizer.
+        :return: List of symbol information.
+        """
+        result = []
+        step = 0
+        while True:
+            line = stdout.readline().decode()
+            if line == "\n":
+                break
+            if step == 0:
+                result.append({"fn": line.strip()})
+                step = 1
+            else:
+                file_name, line, column = line.strip().rsplit(":", 3)
+                result[-1].update({"file": file_name, "column": int(column), "line": int(line)})
+                step = 0
+        return result
+
     def preprocess_frames(
-        self, dbg_path_resolver: PathResolver, trace_doc: Dict[str, Any], input_format: InputFormat
+        self, trace_doc: TraceDoc, input_format: InputFormat
     ) -> List[Dict[str, Any]]:
         """
         Process the paths in frame objects.
 
-        :param dbg_path_resolver: Debug symbols path resolver.
         :param trace_doc: Traceback doc.
         :param input_format: Input format.
         :return: List of traceback frames.
         """
 
         if input_format == InputFormat.CLASSIC:
-            return self.parse_input(trace_doc, dbg_path_resolver)
+            return self.produce_frames(trace_doc)
         elif input_format == InputFormat.THIN:
-            frames = trace_doc["backtrace"]
+            frames = trace_doc.backtrace
             for frame in frames:
-                frame["path"] = dbg_path_resolver.get_dbg_file(frame)
+                soinfo = SoInfo(build_id=frame.get(BUILD_ID_KEY))
+                frame["path"] = self.dbg_path_resolver.get_dbg_file(soinfo)
             return frames
 
     @staticmethod
@@ -695,15 +662,13 @@ class SymbolizerOrchestrator:
 
     def preprocess_frames_with_retries(
         self,
-        dbg_path_resolver: PathResolver,
-        trace_doc: Dict[str, Any],
+        trace_doc: TraceDoc,
         input_format: InputFormat,
         total_seconds_for_retries: int = 0,
     ) -> List[Dict[str, Any]]:
         """
         Process the paths in frame objects.
 
-        :param dbg_path_resolver: Debug symbols path resolver.
         :param trace_doc: Traceback doc.
         :param input_format: Input format.
         :param total_seconds_for_retries: Timeout for retries.
@@ -717,7 +682,7 @@ class SymbolizerOrchestrator:
             retry_error_callback=lambda retry_state: retry_state.outcome.result(),
         )
 
-        return retrying(self.preprocess_frames, dbg_path_resolver, trace_doc, input_format)
+        return retrying(self.preprocess_frames, trace_doc, input_format)
 
     @staticmethod
     def classic_output(frames, outfile, **kwargs):
@@ -744,7 +709,6 @@ class Symbolizer(Subcommand):
         dsym_hint: List[str],
         input_format: str,
         output_format: str,
-        src_dir_to_move: str,
         live: bool,
         host: str,
         cache_dir: str,
@@ -759,7 +723,6 @@ class Symbolizer(Subcommand):
             dsym_hint=dsym_hint,
             input_format=InputFormat.CLASSIC if input_format == "classic" else InputFormat.THIN,
             output_format=OutputFormat.CLASSIC if output_format == "classic" else OutputFormat.JSON,
-            src_dir_to_move=src_dir_to_move,
             live=live,
             host=host,
             cache_dir=cache_dir,
@@ -768,9 +731,21 @@ class Symbolizer(Subcommand):
             client_id=client_id,
         )
 
+        self.dbg_path_resolver = PathResolver(
+            host=self.symbolizer_params.host,
+            cache_dir=self.symbolizer_params.cache_dir,
+            client_secret=self.symbolizer_params.client_secret,
+            client_id=self.symbolizer_params.client_id,
+        )
+
     def execute(self) -> SubcommandResult:
         """Execute symbolize subcommand."""
-        inject.configure()
+
+        def dependencies(binder: inject.Binder) -> None:
+            """Define dependencies for execution."""
+            binder.bind(PathResolver, self.dbg_path_resolver)
+
+        inject.configure(dependencies)
         symbolize_orchestrator = inject.instance(SymbolizerOrchestrator)
 
         success = symbolize_orchestrator.execute(self.symbolizer_params)
@@ -793,7 +768,6 @@ class SymbolizerPlugin(PluginInterface):
             dsym_hint=parsed_args.dsym_hint,
             input_format=parsed_args.input_format,
             output_format=parsed_args.output_format,
-            src_dir_to_move=parsed_args.src_dir_to_move,
             live=parsed_args.live,
             host=parsed_args.host,
             cache_dir=parsed_args.cache_dir,
@@ -826,13 +800,6 @@ class SymbolizerPlugin(PluginInterface):
             default="classic",
             help='"json" shows some extra information',
         )
-        parser.add_argument(
-            "--src-dir-to-move",
-            action="store",
-            type=str,
-            default=None,
-            help="Specify a src dir to move to /data/mci/{original_buildid}/src",
-        )
 
         parser.add_argument("--live", action="store_true", help="Enter live mode.")
 
